#include "WiflyInterface.h"

WiflyInterface::WiflyInterface( PinName tx, PinName rx, PinName reset, PinName tcp_status,
                                const char * ssid, const char * phrase, Security sec) :
    Wifly(tx, rx, reset, tcp_status, ssid, phrase, sec)
{
    ip_set = false;
    mac_set = false;
}

int WiflyInterface::init()
{
    state.dhcp = true;
    ip_set = false;
    reset();
    configure();
    return 0;
}

int WiflyInterface::init(const char* ip, const char* mask, const char* gateway)
{
    state.dhcp = false;
    this->ip = ip;
    strcpy(ip_string, ip);
    ip_set = true;
    this->netmask = mask;
    this->gateway = gateway;
    reset();
    configure();
    return 0;
}

void WiflyInterface::setSecurity(const char * ssid, const char * phrase, Security sec)
{
    Wifly::SetSecurity(ssid, phrase, sec);
}

int WiflyInterface::connect()
{
    return (join()) ? 0 : -1;
}

int WiflyInterface::disconnect()
{
    ip_set = false;
    return Wifly::disconnect();
}

// command   "get mac\r"
// response  "MAC Addr=##:##:##:##:##:##\n<4.40> "
char * WiflyInterface::getMACAddress()
{
    char buffer[50];
    char * match = buffer;
    char * p;
    if (!mac_set) {
        ip_string[0] = '\0';
        if (!sendCommand("get mac\r", NULL, buffer))
            return NULL;
        exit();
        flush();
        // revised to find the mac address string, following a small
        // change where \r are permitted in the captured
        // reply to a command.
        while (*match && (*match != '='))
            match++;
        if (*match) {
            match++;    // Advance past '='
            p = match;
            while (strchr("0123456789abcdefABCDEF:", *p))
                p++;
            *p = '\0';
            if (strlen(match) <= 17) {  // 192.168.101.202
                strcpy(mac_string, match);
                mac_set = true;
            }
        }
    }
    return ip_string;
}

// typical response might be
// \r192.168.43.107\r
// <4.40>
char * WiflyInterface::getIPAddress()
{
    char buffer[50];
    char * match = buffer;
    char * p;
    if (!ip_set) {
        if (!sendCommand("get ip a\r", NULL, buffer))
            return NULL;
        exit();
        flush();
        // revised to find the IP string, following a small
        // change where \r are permitted in the captured
        // reply to a command.
        while (*match && (*match < '0' || *match > '9'))
            match++;
        p = match;
        while (strchr("0123456789.", *p))
            p++;
        *p = '\0';
        if (strlen(match) <= 15) {  // 192.168.101.202
            strcpy(ip_string, match);
            ip_set = true;
        }
    }
    return ip_string;
}


int WiflyInterface::setName(const char * myname)
{
    (void)myname;   // unsupported at this time...
    return 0;       // indicate success...
}

const char * WiflyInterface::getName(void)
{
    return NULL;
}
